import torch
from transformers import AutoTokenizer, CLIPProcessor

from medpalm.transformer import (
    AutoregressiveWrapper,
    Decoder,
    Encoder,
    Transformer,
    ViTransformerWrapper,
)


class MedPalmTokenizer:
    def __init__(self):
        try:
            self.processor = CLIPProcessor.from_pretrained(
                "laion/CLIP-ViT-L-14-laion2B-s32B-b82K"
            )
            self.tokenizer = AutoTokenizer.from_pretrained(
                "EleutherAI/gpt-neox-20b",
                additional_special_tokens=["<image>", "</image>"],
                eos_token="<eos>",
                pad_token="<pad>",
                extra_ids=0,
                model_max_length=8192,
            )

            self.im_idx, self.im_end_idx = self.tokenizer.convert_tokens_to_ids(
                ["<image>", "</image>"]
            )
        except Exception as e:
            print(f"Error init tokenizer: {e}")

    def tokenize_texts(self, texts):
        try:
            texts = self.tokenizer(
                texts, return_tensors="pt", padding=True, truncation=True
            ).input_ids
            image_tokens = torch.tensor(
                [[self.im_idx, self.im_end_idx]] * texts.shape[0]
            )
            return torch.cat([texts[:, 0:1], image_tokens, texts[:, 1:]], dim=1), texts
        except Exception as e:
            print(f"Error tokenizing texts: {e}")

    def tokenize_images(self, images):
        try:
            tokenized_images = self.processor(
                images=images, return_tensors="pt"
            ).pixel_values
            print(f"Tokenized image: {tokenized_images.shape}")
            return tokenized_images

        except Exception as e:
            print(f"Error tokenizing texts: {e}")

    def tokenize(self, sample):
        try:
            text_tokens, only_text_tokens = self.tokenize_texts(sample["target_text"])
            attention_mask = text_tokens != self.tokenizer.pad_token_id
            dummy_image_features = torch.ones((text_tokens.shape[0], 64))
            attention_mask = torch.cat([dummy_image_features, attention_mask], dim=1)
            return {
                "text_tokens": text_tokens,
                "images": self.tokenize_images(sample["image"]),
                "labels": only_text_tokens,
                "attention_mask": attention_mask,
            }

        except Exception as e:
            print(f"Error during tokenization {e}")


class MedPalm(torch.nn.Module):
    """
    MedPalm model for medical image and text processing.

    Args:
        image_size (int): Size of the input image (default: 256).
        patch_size (int): Size of each image patch (default: 32).
        encoder_dim (int): Dimensionality of the encoder (default: 512).
        encoder_depth (int): Number of encoder layers (default: 6).
        encoder_heads (int): Number of attention heads in the encoder (default: 8).
        num_tokens (int): Number of tokens in the decoder (default: 20000).
        max_seq_len (int): Maximum sequence length in the decoder (default: 1024).
        decoder_dim (int): Dimensionality of the decoder (default: 512).
        decoder_depth (int): Number of decoder layers (default: 6).
        decoder_heads (int): Number of attention heads in the decoder (default: 8).
        alibi_num_heads (int): Number of attention heads in the alibi mechanism (default: 4).
        use_abs_pos_emb (bool): Whether to use absolute positional embeddings (default: False).
        cross_attend (bool): Whether to enable cross-attention in the decoder (default: True).
        alibi_pos_bias (bool): Whether to use positional bias in the alibi mechanism (default: True).
        rotary_xpos (bool): Whether to use rotary positional embeddings (default: True).
        attn_flash (bool): Whether to use attention flash in the decoder (default: True).
        qk_norm (bool): Whether to normalize the query-key vectors in attention (default: True).
    """

    def __init__(
        self,
        image_size=256,
        patch_size=32,
        encoder_dim=512,
        encoder_depth=6,
        encoder_heads=8,
        num_tokens=20000,
        max_seq_len=1024,
        decoder_dim=512,
        decoder_depth=6,
        decoder_heads=8,
        alibi_num_heads=4,
        use_abs_pos_emb=False,
        cross_attend=True,
        alibi_pos_bias=True,
        rotary_xpos=True,
        attn_flash=True,
        qk_norm=True,
    ):
        super(MedPalm, self).__init__()

        self.encoder = ViTransformerWrapper(
            image_size=image_size,
            patch_size=patch_size,
            attn_layers=Encoder(
                dim=encoder_dim, depth=encoder_depth, heads=encoder_heads
            ),
        )

        self.decoder = Transformer(
            num_tokens=num_tokens,
            max_seq_len=max_seq_len,
            use_abs_pos_emb=use_abs_pos_emb,
            attn_layers=Decoder(
                dim=decoder_dim,
                depth=decoder_depth,
                heads=decoder_heads,
                cross_attend=cross_attend,
                alibi_pos_bias=alibi_pos_bias,
                alibi_num_heads=alibi_num_heads,
                rotary_xpos=rotary_xpos,
                attn_flash=attn_flash,
                qk_norm=qk_norm,
            ),
        )

        self.decoder = AutoregressiveWrapper(self.decoder)

    def forward(self, img, text):
        """
        Forward pass of the MedPalm model.

        Args:
            img (torch.Tensor): Input image tensor.
            text (torch.Tensor): Input text tensor.

        Returns:
            torch.Tensor: Output tensor from the decoder.
        """
        try:
            encoded = self.encoder(img, return_embeddings=True)
            return self.decoder(text, context=encoded)
        except Exception as error:
            print(f"Failed in forward method: {error}")
            raise



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['medpalm']

package_data = \
{'': ['*']}

install_requires = \
['SentencePiece', 'bitsandbytes', 'datasets', 'einops', 'torch', 'transformers']

setup_kwargs = {
    'name': 'medpalm',
    'version': '0.1.7',
    'description': 'MedPalm - Pytorch',
    'long_description': "[![Multi-Modality](agorabanner.png)](https://discord.gg/qUtxnK2NMf)\n\n\n# Med-PaLM \nA responsible path to generative AI in healthcare: Unleash the power of Med-PaLM 2 to revolutionize medical knowledge, answer complex questions, and enhance healthcare experiences with accuracy, safety, and equitable practices.\n\n[![GitHub issues](https://img.shields.io/github/issues/kyegomez/Med-Palm)](https://github.com/kyegomez/Med-Palm/issues) \n[![GitHub forks](https://img.shields.io/github/forks/kyegomez/Med-Palm)](https://github.com/kyegomez/Med-Palm/network) \n[![GitHub stars](https://img.shields.io/github/stars/kyegomez/Med-Palm)](https://github.com/kyegomez/Med-Palm/stargazers) [![GitHub license](https://img.shields.io/github/license/kyegomez/Med-Palm)](https://github.com/kyegomez/Med-Palm/blob/master/LICENSE)\n[![Share on Twitter](https://img.shields.io/twitter/url/https/twitter.com/cloudposse.svg?style=social&label=Share%20%40kyegomez/Med-Palm)](https://twitter.com/intent/tweet?text=Excited%20to%20introduce%20Med-Palm,%20the%20all-new%20robotics%20model%20with%20the%20potential%20to%20revolutionize%20automation.%20Join%20us%20on%20this%20journey%20towards%20a%20smarter%20future.%20%23RT1%20%23Robotics&url=https%3A%2F%2Fgithub.com%2Fkyegomez%2FMed-Palm)\n[![Share on Facebook](https://img.shields.io/badge/Share-%20facebook-blue)](https://www.facebook.com/sharer/sharer.php?u=https%3A%2F%2Fgithub.com%2Fkyegomez%2FMed-Palm)\n[![Share on LinkedIn](https://img.shields.io/badge/Share-%20linkedin-blue)](https://www.linkedin.com/shareArticle?mini=true&url=https%3A%2F%2Fgithub.com%2Fkyegomez%2FMed-Palm&title=Introducing%20Med-Palm%2C%20the%20All-New%20Robotics%20Model&summary=Med-Palm%20is%20the%20next-generation%20robotics%20model%20that%20promises%20to%20transform%20industries%20with%20its%20intelligence%20and%20efficiency.%20Join%20us%20to%20be%20a%20part%20of%20this%20revolutionary%20journey%20%23RT1%20%23Robotics&source=)\n![Discord](https://img.shields.io/discord/999382051935506503)\n[![Share on Reddit](https://img.shields.io/badge/-Share%20on%20Reddit-orange)](https://www.reddit.com/submit?url=https%3A%2F%2Fgithub.com%2Fkyegomez%2FMed-Palm&title=Exciting%20Times%20Ahead%20with%20Med-Palm%2C%20the%20All-New%20Robotics%20Model%20%23RT1%20%23Robotics) [![Share on Hacker News](https://img.shields.io/badge/-Share%20on%20Hacker%20News-orange)](https://news.ycombinator.com/submitlink?u=https%3A%2F%2Fgithub.com%2Fkyegomez%2FMed-Palm&t=Exciting%20Times%20Ahead%20with%20Med-Palm%2C%20the%20All-New%20Robotics%20Model%20%23RT1%20%23Robotics)\n[![Share on Pinterest](https://img.shields.io/badge/-Share%20on%20Pinterest-red)](https://pinterest.com/pin/create/button/?url=https%3A%2F%2Fgithub.com%2Fkyegomez%2FMed-Palm&media=https%3A%2F%2Fexample.com%2Fimage.jpg&description=Med-Palm%2C%20the%20Revolutionary%20Robotics%20Model%20that%20will%20Change%20the%20Way%20We%20Work%20%23RT1%20%23Robotics)\n[![Share on WhatsApp](https://img.shields.io/badge/-Share%20on%20WhatsApp-green)](https://api.whatsapp.com/send?text=I%20just%20discovered%20Med-Palm,%20the%20all-new%20robotics%20model%20that%20promises%20to%20revolutionize%20automation.%20Join%20me%20on%20this%20exciting%20journey%20towards%20a%20smarter%20future.%20%23RT1%20%23Robotics%0A%0Ahttps%3A%2F%2Fgithub.com%2Fkyegomez%2FMed-Palm)\n\n\n![Med palm](image-9.png)\n\n## Getting Started\n\n```\npip install MedPalm\n```\n\n## Usage\n\n\n```python\nimport torch\nfrom medpalm.model import MedPalm\n\n#usage\nimg = torch.randn(1, 3, 256, 256)\ntext = torch.randint(0, 20000, (1, 4096))\n\nmodel = MedPalm()\noutput = model(img, text)\nprint(output.shape)\n```\n📝 Note: Modify the examples to suit your data and project needs.\n\n## 📚 Datasets \nHere is a comprehensive markdown table containing metadata and details for all the datasets mentioned in the MED PALM paper\n\n| Dataset | Modality | Description | # Training examples | # Test examples | Tasks |\n|-|-|-|-|-|-|\n| MultiMedBench | Multimodal | Benchmark for biomedical AI | | | 14 biomedical tasks |\n| MedQA | Text | US licensing exam questions | 10,178 | 1,273 | Question answering |\n| MedMCQA | Text | Indian medical exam questions | 182,822 | 4,183 | Question answering |\n| PubMedQA | Text | Biomedical literature questions | 0 | 500 | Question answering |  \n| MIMIC-III | Radiology reports | Radiology reports for ICU patients | 58,405 reports | 13,057 reports | Report summarization |\n| VQA-RAD | Radiology images | QA pairs on radiology images | 1,797 QA pairs | 451 QA pairs | Visual question answering |\n| Slake-VQA | Radiology images | English-Chinese QA pairs | 9,849 samples | 2,070 samples | Visual question answering |\n| Path-VQA | Pathology images | QA pairs on pathology images | 19,755 QA pairs | 6,761 QA pairs | Visual question answering |\n| MIMIC-CXR | Chest X-ray | Images and reports | 353,542 | 4,834 | Report generation, classification |  \n| PAD-UFES-20 | Dermatology images | Skin lesion smartphone images | 1,838 images | 460 images | Image classification |\n| CBIS-DDSM (mass) | Mammography | Mammogram mass patches | 1,318 images | 378 images | Image classification |\n| CBIS-DDSM (calcification) | Mammography | Mammogram calcification patches | 1,544 images | 326 images | Image classification |\n| VinDr-Mammo | Mammography | Mammogram studies | 16,000 images | 4,000 images | Image classification |\n| PrecisionFDA (training) | Genomics | Genomic variant images | 197,038 images | | Image classification |\n| PrecisionFDA (evaluation) | Genomics | Genomic variant images | | 13,030 images | Image classification |\n| Montgomery County | Chest X-ray | Chest X-rays | 0 | 138 images | TB detection evaluation |  \n| MIMIC-CXR (human evaluation) | Chest X-ray | Chest X-ray images and reports | | 246 cases | Human evaluation |\n\n---\n\n## 💼 Commercial Use-Cases\n\nMed Palm has thousands of potential use cases the 3 below are simple, for more detailed applications check out my new blog article on MedPalm's use in the real world. [Click here to learn more](https://medium.com/@kyeg/how-medpalm-is-revolutionizing-medicine-62eef979f0e5)\n\n- **Clinical Diagnostics**: Combining medical imaging, patient tales 📖, and genes, we're aiming for top-notch diagnostic solutions.\n  \n- **Healthcare Research**: Dive deep into diverse datasets and discover something new with Med-PaLM by your side! 🤿\n  \n- **Telemedicine**: Quick, reliable, and remote! 🌍 Med-PaLM's here to revolutionize telehealth.\n\n# Contributing to Med Palm 🤖🌟\nHelp with the todo list!\n\n----\n\n## License\n\nMed-PaLM's is under the MIT license. Check out the details [here](LICENSE.md).\n\n## Citation\n```bibtex\n@misc{2307.14334,\nAuthor = {Tao Tu and Shekoofeh Azizi and Danny Driess and Mike Schaekermann and Mohamed Amin and Pi-Chuan Chang and Andrew Carroll and Chuck Lau and Ryutaro Tanno and Ira Ktena and Basil Mustafa and Aakanksha Chowdhery and Yun Liu and Simon Kornblith and David Fleet and Philip Mansfield and Sushant Prakash and Renee Wong and Sunny Virmani and Christopher Semturs and S Sara Mahdavi and Bradley Green and Ewa Dominowska and Blaise Aguera y Arcas and Joelle Barral and Dale Webster and Greg S. Corrado and Yossi Matias and Karan Singhal and Pete Florence and Alan Karthikesalingam and Vivek Natarajan},\nTitle = {Towards Generalist Biomedical AI},\nYear = {2023},\nEprint = {arXiv:2307.14334},\n}\n```\n\n## Todo\n- [ ] Implement a training script using Huggingface's accelerate and deepspeed\n- [",
    'author': 'Kye Gomez',
    'author_email': 'kye@apac.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/kyegomez/med-palm',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
